searchNodes=[{"ref":"Membrane.Pipeline.html","title":"Membrane.Pipeline","type":"behaviour","doc":"Module containing functions for constructing and supervising pipelines. Pipelines are units that make it possible to instantiate, link and manage elements in convenient way (actually elements should always be used inside a pipeline). Linking elements together enables them to pass data to one another, and process it in different ways."},{"ref":"Membrane.Pipeline.html#change_playback_state/2","title":"Membrane.Pipeline.change_playback_state/2","type":"function","doc":"Changes the playback state to the new_state."},{"ref":"Membrane.Pipeline.html#child_spec/1","title":"Membrane.Pipeline.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Membrane.Pipeline.html#c:handle_init/1","title":"Membrane.Pipeline.handle_init/1","type":"callback","doc":"Callback invoked on initialization of pipeline process. It should parse options and initialize element internal state. Internally it is invoked inside GenServer.init/1 callback."},{"ref":"Membrane.Pipeline.html#c:handle_notification/3","title":"Membrane.Pipeline.handle_notification/3","type":"callback","doc":"Callback invoked when a notification comes in from an element."},{"ref":"Membrane.Pipeline.html#c:handle_other/2","title":"Membrane.Pipeline.handle_other/2","type":"callback","doc":"Callback invoked when pipeline receives a message that is not recognized as an internal membrane message. Useful for receiving ticks from timer, data sent from NIFs or other stuff."},{"ref":"Membrane.Pipeline.html#c:handle_playing_to_prepared/1","title":"Membrane.Pipeline.handle_playing_to_prepared/1","type":"callback","doc":"Callback invoked when pipeline transition from :playing to :prepared state has finished, that is all of its elements are prepared to be stopped."},{"ref":"Membrane.Pipeline.html#c:handle_prepared_to_playing/1","title":"Membrane.Pipeline.handle_prepared_to_playing/1","type":"callback","doc":"Callback invoked when pipeline is in :playing state, i.e. all its elements are in this state."},{"ref":"Membrane.Pipeline.html#c:handle_prepared_to_stopped/1","title":"Membrane.Pipeline.handle_prepared_to_stopped/1","type":"callback","doc":"Callback invoked when pipeline is in :playing state, i.e. all its elements are in this state."},{"ref":"Membrane.Pipeline.html#c:handle_spec_started/2","title":"Membrane.Pipeline.handle_spec_started/2","type":"callback","doc":"Callback invoked when Membrane.Pipeline.Spec is linked and in the same playback state as pipeline. Spec can be started from handle_init/1 callback or as spec_action_t/0 action."},{"ref":"Membrane.Pipeline.html#c:handle_stopped_to_prepared/1","title":"Membrane.Pipeline.handle_stopped_to_prepared/1","type":"callback","doc":"Callback invoked when pipeline transition from :stopped to :prepared state has finished, that is all of its elements are prepared to enter :playing state."},{"ref":"Membrane.Pipeline.html#c:membrane_pipeline?/0","title":"Membrane.Pipeline.membrane_pipeline?/0","type":"callback","doc":"Enables to check whether module is membrane pipeline"},{"ref":"Membrane.Pipeline.html#pipeline?/1","title":"Membrane.Pipeline.pipeline?/1","type":"function","doc":"Checks whether module is a pipeline."},{"ref":"Membrane.Pipeline.html#play/1","title":"Membrane.Pipeline.play/1","type":"function","doc":"Changes playback state to :playing. An alias for change_playback_state/2 with proper state."},{"ref":"Membrane.Pipeline.html#prepare/1","title":"Membrane.Pipeline.prepare/1","type":"function","doc":"Changes playback state to :prepared. An alias for change_playback_state/2 with proper state."},{"ref":"Membrane.Pipeline.html#start/3","title":"Membrane.Pipeline.start/3","type":"function","doc":"Does the same as start_link/3 but starts process outside of supervision tree."},{"ref":"Membrane.Pipeline.html#start_link/3","title":"Membrane.Pipeline.start_link/3","type":"function","doc":"Starts the Pipeline based on given module and links it to the current process. Pipeline options are passed to module&#39;s handle_init/1 callback. Process options are internally passed to GenServer.start_link/3. Returns the same values as GenServer.start_link/3."},{"ref":"Membrane.Pipeline.html#stop/1","title":"Membrane.Pipeline.stop/1","type":"function","doc":"Changes playback state to :stopped. An alias for change_playback_state/2 with proper state."},{"ref":"Membrane.Pipeline.html#stop_and_terminate/1","title":"Membrane.Pipeline.stop_and_terminate/1","type":"function","doc":"Changes pipeline&#39;s playback state to :stopped and terminates its process"},{"ref":"Membrane.Pipeline.html#t:action_t/0","title":"Membrane.Pipeline.action_t/0","type":"type","doc":"Type describing actions that can be returned from pipeline callbacks. Returning actions is a way of pipeline interaction with its elements and other parts of framework."},{"ref":"Membrane.Pipeline.html#t:callback_return_t/0","title":"Membrane.Pipeline.callback_return_t/0","type":"type","doc":"Type that defines all valid return values from most callbacks."},{"ref":"Membrane.Pipeline.html#t:forward_action_t/0","title":"Membrane.Pipeline.forward_action_t/0","type":"type","doc":"Action that sends a message to element identified by name."},{"ref":"Membrane.Pipeline.html#t:pipeline_options_t/0","title":"Membrane.Pipeline.pipeline_options_t/0","type":"type","doc":"Defines options that can be passed to start/3 / start_link/3 and received in handle_init/1 callback."},{"ref":"Membrane.Pipeline.html#t:playback_state_t/0","title":"Membrane.Pipeline.playback_state_t/0","type":"type","doc":""},{"ref":"Membrane.Pipeline.html#t:remove_child_action_t/0","title":"Membrane.Pipeline.remove_child_action_t/0","type":"type","doc":"Action that stops, unlinks and removes specified child/children from pipeline."},{"ref":"Membrane.Pipeline.html#t:spec_action_t/0","title":"Membrane.Pipeline.spec_action_t/0","type":"type","doc":"Action that instantiates elements and links them according to Membrane.Pipeline.Spec. Elements playback state is changed to the current pipeline state. c:handle_spec_started callback is executed once it happens."},{"ref":"Membrane.Pipeline.Spec.html","title":"Membrane.Pipeline.Spec","type":"module","doc":"Structure representing topology of a pipeline. It can be returned from Membrane.Pipeline.handle_init/1 callback upon pipeline&#39;s initialization. It will define a topology of children and links that build the pipeline. Children Children that should be spawned when the pipeline starts can be defined with the :children field. You have to set it to a keyword list, where keys are valid element names (Membrane.Element.name_t/0) that are unique within this pipeline and values are either element&#39;s module or struct of that module. Sample definitions: [ first_element: %Element.With.Options.Struct{option_a: 42}, some_element: Element.Without.Options, other_element: Element.Using.Default.Options ] Links Links that should be made when the pipeline starts, and children are spawned can be defined with the :links field. You have to set it to a map, where both keys and values are tuples of {element_name, pad_name}. Entries can also have additional options passed by keyword list at the end of a tuple (See endpoint_options_t/0). Element names have to match names given to the :children field. Once it&#39;s done, pipeline will ensure that links are present. Sample definition: %{ {:source_a, :output} =&gt; {:converter, :input, buffer: [preferred_size: 20_000]}, {:converter, :output} =&gt; {:mixer, :input_a}, {:source_b, :output} =&gt; {:mixer, :input_b, pad: [mute: true]} {:mixer, :output} =&gt; {:sink, :input, buffer: [warn_size: 264_000, fail_size: 300_000]}, }"},{"ref":"Membrane.Pipeline.Spec.html#t:child_spec_t/0","title":"Membrane.Pipeline.Spec.child_spec_t/0","type":"type","doc":""},{"ref":"Membrane.Pipeline.Spec.html#t:children_spec_t/0","title":"Membrane.Pipeline.Spec.children_spec_t/0","type":"type","doc":"Description of all the children elements inside the pipeline"},{"ref":"Membrane.Pipeline.Spec.html#t:endpoint_options_t/0","title":"Membrane.Pipeline.Spec.endpoint_options_t/0","type":"type","doc":"Options passed to the element when linking its pad with different one. The allowed options are: :buffer - keywoed allowing to configure Membrane.Core.InputBuffer between elements. Valid only for input pads. See Membrane.Core.InputBuffer.props_t/0 for configurable properties. :pad - any element-specific options that will be available in Membrane.Element.Pad.Data struct."},{"ref":"Membrane.Pipeline.Spec.html#t:link_endpoint_spec_t/0","title":"Membrane.Pipeline.Spec.link_endpoint_spec_t/0","type":"type","doc":"Spec for one of the ends of the link"},{"ref":"Membrane.Pipeline.Spec.html#t:links_spec_t/0","title":"Membrane.Pipeline.Spec.links_spec_t/0","type":"type","doc":"Map describing links between elements"},{"ref":"Membrane.Pipeline.Spec.html#t:t/0","title":"Membrane.Pipeline.Spec.t/0","type":"type","doc":"Struct used when launching a pipeline"},{"ref":"Membrane.Core.InputBuffer.html","title":"Membrane.Core.InputBuffer","type":"module","doc":"Buffer that is attached to the :input pad when working in a :pull mode. It stores Membrane.Buffer, Membrane.Event and Membrane.Caps structs and prevents the situation where the data in a stream contains the discontinuities. It also guarantees that element won&#39;t be flooded with the incoming data."},{"ref":"Membrane.Core.InputBuffer.html#empty?/1","title":"Membrane.Core.InputBuffer.empty?/1","type":"function","doc":""},{"ref":"Membrane.Core.InputBuffer.html#new/6","title":"Membrane.Core.InputBuffer.new/6","type":"function","doc":""},{"ref":"Membrane.Core.InputBuffer.html#parse_props/1","title":"Membrane.Core.InputBuffer.parse_props/1","type":"function","doc":""},{"ref":"Membrane.Core.InputBuffer.html#store/3","title":"Membrane.Core.InputBuffer.store/3","type":"function","doc":""},{"ref":"Membrane.Core.InputBuffer.html#take/2","title":"Membrane.Core.InputBuffer.take/2","type":"function","doc":""},{"ref":"Membrane.Core.InputBuffer.html#t:props_t/0","title":"Membrane.Core.InputBuffer.props_t/0","type":"type","doc":"Properties that can be passed when creating new InputBuffer Available options are: :preffered_size - size which will be the &#39;target&#39; for InputBuffer - it will make demands trying to grow to this size. Its default value depends on the set Membrane.Buffer.Metric and is obtained via Membrane.Buffer.Metric.pullbuffer_preferred_size/0 :min_demand - the minimal size of a demand that can be sent to the linked output pad. This prevents from excessive message passing between elements. Defaults to a quarter of preferred size. warn_size - in toilet mode (connecting push output to pull input pad), receiving more data than this size triggers a warning. By default it is equal to twice the preffered size. fail_size - in toilet mode (connecting push output to pull input pad), receiving more data than this results in an element failure. By default, it is four times the preffered size."},{"ref":"Membrane.Core.InputBuffer.html#t:t/0","title":"Membrane.Core.InputBuffer.t/0","type":"type","doc":""},{"ref":"Membrane.Element.html","title":"Membrane.Element","type":"module","doc":"Module containing functions spawning, shutting down, inspecting and controlling playback of elements. These functions are usually called by Membrane.Pipeline, and can be called from elsewhere only if there is a really good reason for doing so."},{"ref":"Membrane.Element.html#change_playback_state/2","title":"Membrane.Element.change_playback_state/2","type":"function","doc":"Changes the playback state to the new_state."},{"ref":"Membrane.Element.html#child_spec/1","title":"Membrane.Element.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Membrane.Element.html#element?/1","title":"Membrane.Element.element?/1","type":"function","doc":"Checks whether module is an element."},{"ref":"Membrane.Element.html#handle_linking_finished/2","title":"Membrane.Element.handle_linking_finished/2","type":"function","doc":"Sends synchronous call to element, informing it that linking has finished."},{"ref":"Membrane.Element.html#handle_new_pad/4","title":"Membrane.Element.handle_new_pad/4","type":"function","doc":"Sends synchronous call to element, requesting it to create a new instance of :on_request pad."},{"ref":"Membrane.Element.html#is_element_name/1","title":"Membrane.Element.is_element_name/1","type":"macro","doc":"Checks whether the given term is a valid element name"},{"ref":"Membrane.Element.html#link/1","title":"Membrane.Element.link/1","type":"function","doc":"Sends synchronous calls to two elements, telling them to link with each other."},{"ref":"Membrane.Element.html#play/1","title":"Membrane.Element.play/1","type":"function","doc":"Changes playback state to :playing. An alias for change_playback_state/2 with proper state."},{"ref":"Membrane.Element.html#prepare/1","title":"Membrane.Element.prepare/1","type":"function","doc":"Changes playback state to :prepared. An alias for change_playback_state/2 with proper state."},{"ref":"Membrane.Element.html#set_controlling_pid/3","title":"Membrane.Element.set_controlling_pid/3","type":"function","doc":"Sends synchronous call to the given element requesting it to set controlling pid. It will wait for reply for amount of time passed as second argument (in milliseconds)."},{"ref":"Membrane.Element.html#set_watcher/3","title":"Membrane.Element.set_watcher/3","type":"function","doc":"Sends synchronous call to the given element requesting it to set watcher. It will wait for reply for amount of time passed as second argument (in milliseconds)."},{"ref":"Membrane.Element.html#shutdown/2","title":"Membrane.Element.shutdown/2","type":"function","doc":"Stops given element process. It will wait for reply for amount of time passed as second argument (in milliseconds). Will trigger calling Membrane.Element.Base.Mixin.CommonBehaviour.handle_shutdown/1 callback."},{"ref":"Membrane.Element.html#start/3","title":"Membrane.Element.start/3","type":"function","doc":"Works similarly to start_link/3, but does not link to the current process."},{"ref":"Membrane.Element.html#start/5","title":"Membrane.Element.start/5","type":"function","doc":"Works similarly to start_link/5, but does not link to the current process."},{"ref":"Membrane.Element.html#start_link/3","title":"Membrane.Element.start_link/3","type":"function","doc":"Works similarly to start_link/5, but passes element struct (with default values) as element options. If element does not define struct, nil is passed."},{"ref":"Membrane.Element.html#start_link/5","title":"Membrane.Element.start_link/5","type":"function","doc":"Starts process for element of given module, initialized with given options and links it to the current process in the supervision tree. Calls GenServer.start_link/3 underneath."},{"ref":"Membrane.Element.html#stop/1","title":"Membrane.Element.stop/1","type":"function","doc":"Changes playback state to :stopped. An alias for change_playback_state/2 with proper state."},{"ref":"Membrane.Element.html#unlink/2","title":"Membrane.Element.unlink/2","type":"function","doc":"Sends synchronous call to element, telling it to unlink all its pads."},{"ref":"Membrane.Element.html#t:name_t/0","title":"Membrane.Element.name_t/0","type":"type","doc":"Type that defines an element name by which it is identified."},{"ref":"Membrane.Element.html#t:options_t/0","title":"Membrane.Element.options_t/0","type":"type","doc":"Defines options that can be passed to start/5 / start_link/5 and received in Membrane.Element.Base.Mixin.CommonBehaviour.handle_init/1 callback."},{"ref":"Membrane.Element.html#t:playback_state_t/0","title":"Membrane.Element.playback_state_t/0","type":"type","doc":""},{"ref":"Membrane.Element.html#t:state_t/0","title":"Membrane.Element.state_t/0","type":"type","doc":"Type of user-managed state of element."},{"ref":"Membrane.Element.html#t:type_t/0","title":"Membrane.Element.type_t/0","type":"type","doc":"Defines possible element types: source, producing buffers filter, processing buffers sink, consuming buffers"},{"ref":"Membrane.Element.Action.html","title":"Membrane.Element.Action","type":"module","doc":"This module contains type specifications of actions that can be returned from element callbacks. Returning actions is a way of element interaction with other elements and parts of framework. Each action may be returned by any callback (except for Membrane.Element.Base.Mixin.CommonBehaviour.handle_init/1 and Membrane.Element.Base.Mixin.CommonBehaviour.handle_shutdown/1, as they do not return any actions) unless explicitly stated otherwise."},{"ref":"Membrane.Element.Action.html#t:buffer_t/0","title":"Membrane.Element.Action.buffer_t/0","type":"type","doc":"Sends buffers through a pad. The pad must have output direction. Allowed only when playback state is playing."},{"ref":"Membrane.Element.Action.html#t:caps_t/0","title":"Membrane.Element.Action.caps_t/0","type":"type","doc":"Sends caps through a pad. The pad must have output direction. Sent caps must fit constraints on the pad. Forbidden when playback state is stopped."},{"ref":"Membrane.Element.Action.html#t:demand_size_t/0","title":"Membrane.Element.Action.demand_size_t/0","type":"type","doc":""},{"ref":"Membrane.Element.Action.html#t:demand_t/0","title":"Membrane.Element.Action.demand_t/0","type":"type","doc":"Makes a demand on a pad. The pad must have input direction and work in pull mode. This action does NOT entail sending demand through the pad, but just requesting some amount of data from Membrane.Core.InputBuffer, which sends demands automatically when it runs out of data. If there is any data available at the pad, the data is passed to Membrane.Element.Base.Filter.handle_process_list/4 or Membrane.Element.Base.Sink.handle_write_list/4 callback. Invoked callback is guaranteed not to receive more data than demanded. Demand size can be either a non-negative integer, that overrides existing demand, or a function that is passed current demand, and is to return the new demand. Allowed only when playback state is playing."},{"ref":"Membrane.Element.Action.html#t:event_t/0","title":"Membrane.Element.Action.event_t/0","type":"type","doc":"Sends an event through a pad (input or output). Forbidden when playback state is stopped."},{"ref":"Membrane.Element.Action.html#t:forward_t/0","title":"Membrane.Element.Action.forward_t/0","type":"type","doc":"Sends buffers/caps/event to all output pads of element (or to input pads when event occurs on the output pad). Used by default implementations of Membrane.Element.Base.Mixin.SinkBehaviour.handle_caps/4 and Membrane.Element.Base.Mixin.CommonBehaviour.handle_event/4 callbacks in filter. Allowed only when all below conditions are met: element is filter, callback is Membrane.Element.Base.Filter.handle_process_list/4, Membrane.Element.Base.Mixin.SinkBehaviour.handle_caps/4 or Membrane.Element.Base.Mixin.CommonBehaviour.handle_event/4, playback state is valid for sending buffer, caps or event action respectively. Keep in mind that Membrane.Element.Base.Filter.handle_process_list/4 can only forward buffers, Membrane.Element.Base.Mixin.SinkBehaviour.handle_caps/4 - caps and Membrane.Element.Base.Mixin.CommonBehaviour.handle_event/4 - events."},{"ref":"Membrane.Element.Action.html#t:notify_t/0","title":"Membrane.Element.Action.notify_t/0","type":"type","doc":"Sends a message to the watcher."},{"ref":"Membrane.Element.Action.html#t:playback_change_t/0","title":"Membrane.Element.Action.playback_change_t/0","type":"type","doc":"Suspends/resumes change of playback state. playback_change: :suspend may be returned only from Membrane.Element.Base.Mixin.CommonBehaviour.handle_stopped_to_prepared/2, Membrane.Element.Base.Mixin.CommonBehaviour.handle_playing_to_prepared/2, Membrane.Element.Base.Mixin.CommonBehaviour.handle_prepared_to_playing/2 and Membrane.Element.Base.Mixin.CommonBehaviour.handle_prepared_to_stopped/2 callbacks, and defers playback state change until playback_change: :resume is returned. playback_change: :resume may be returned from any callback, only when playback state change is suspended, and causes it to finish. There is no straight limit how long playback change can take, but keep in mind that it may affect application quality if not done quick enough."},{"ref":"Membrane.Element.Action.html#t:redemand_t/0","title":"Membrane.Element.Action.redemand_t/0","type":"type","doc":"Executes Membrane.Element.Base.Mixin.SourceBehaviour.handle_demand/5 callback for the given pad if its demand is greater than 0. The pad must have output direction and work in pull mode. Redemand in Sources In case of Sources, :redemand is just a helper that simplifies element&#39;s code. The element doesn&#39;t need to generate the whole demand synchronously at handle_demand or store current demand size in its state, but it can just generate one buffer and return :redemand action. If there is still one or more buffers to produce, returning :redemand triggers the next invocation of handle_demand. In such case, the element is to produce next buffer and call :redemand again. If there are no more buffers demanded, handle_demand is not invoked and the loop ends. One more advantage of the approach with :redemand action is that produced buffers are sent one after another in separate messages and this can possibly improve the latency. Redemand in Filters Redemand in Filters is useful in a situation where not the entire demand of output pad has been satisfied and there is a need to send a demand for additional buffers through the input pad. A typical example of this situation is a parser that has not demanded enough bytes to parse the whole frame. Usage limitations Allowed only when playback state is playing."},{"ref":"Membrane.Element.Action.html#t:split_t/0","title":"Membrane.Element.Action.split_t/0","type":"type","doc":"Allows to split callback execution into multiple applications of another callback (called from now sub-callback). Executions are synchronous in the element process, and each of them passes subsequent arguments from the args_list, along with the element state (passed as the last argument each time). Return value of each execution of sub-callback can be any valid return value of the original callback (this also means sub-callback can return any action valid for the original callback, unless expliciltly stated). Returned actions are executed immediately (they are NOT accumulated and executed after all sub-callback executions are finished). Useful when a long action is to be undertaken, and partial results need to be returned before entire process finishes (e.g. default implementation of Membrane.Element.Base.Filter.handle_process_list/4 uses split action to invoke Membrane.Element.Base.Filter.handle_process/4 with each buffer)"},{"ref":"Membrane.Element.Action.html#t:t/0","title":"Membrane.Element.Action.t/0","type":"type","doc":"Type that defines a single action that may be returned from element callbacks. Depending on element type, callback, current playback state and other circumstances there may be different actions available."},{"ref":"Membrane.Element.Base.html","title":"Membrane.Element.Base","type":"module","doc":"Modules in this namespace contain behaviours, default callback implementations and other stuff useful when creating elements. Elements are units that produce, process or consume data. They can be linked with Membrane.Pipeline, and thus form a pipeline able to perform complex data processing. Each element defines a set of pads, through which it can be linked with other elements. During playback, pads can either send (output pads) or receive (input pads) data. For more information on pads, see Membrane.Element.Pad. To implement an element, one of base modules (Membrane.Element.Base.Source, Membrane.Element.Base.Filter, Membrane.Element.Base.Sink) has to be used, depending on the element type: source, producing buffers (contain only output pads), filter, processing buffers (contain both input and output pads), sink, consuming buffers (contain only input pads). For more information on each element type, check documentation for appropriate base module. Behaviours Element-specific behaviours are specified in modules: Membrane.Element.Base.Mixin.CommonBehaviour - behaviour common to all elements, Membrane.Element.Base.Mixin.SourceBehaviour - behaviour common to sources and filters, Membrane.Element.Base.Mixin.SinkBehaviour - behaviour common to sinks and filters, Base modules (Membrane.Element.Base.Source, Membrane.Element.Base.Filter, Membrane.Element.Base.Sink) - behaviours specific to each element type. Callbacks Modules listed above provide specifications of callbacks that define elements lifecycle. All of these callbacks have names with the handle_ prefix. They are used to define reaction to certain events that happen during runtime, and indicate what actions frawork should undertake as a result, besides executing element-specific code. For actions that can be returned by each callback, see Membrane.Element.Action module."},{"ref":"Membrane.Element.Base.Filter.html","title":"Membrane.Element.Base.Filter","type":"behaviour","doc":"Module defining behaviour for filters - elements processing data. Behaviours for filters are specified, besides this place, in modules Membrane.Element.Base.Mixin.CommonBehaviour, Membrane.Element.Base.Mixin.SourceBehaviour, and Membrane.Element.Base.Mixin.SinkBehaviour. Filters can have both input and output pads. Job of a usual filter is to receive some data on a input pad, process the data and send it through the output pad. If these pads work in pull mode, which is the most common case, then filter is also responsible for receiving demands on the output pad and requesting them on the input pad (for more details, see Membrane.Element.Base.Mixin.SourceBehaviour.handle_demand/5 callback). Filters, like all elements, can of course have multiple pads if needed to provide more complex solutions."},{"ref":"Membrane.Element.Base.Filter.html#c:handle_process/4","title":"Membrane.Element.Base.Filter.handle_process/4","type":"callback","doc":"Callback that is to process buffers. In contrast to handle_process_list/4, it is passed only a single buffer. Called by default implementation of handle_process_list/4."},{"ref":"Membrane.Element.Base.Filter.html#c:handle_process_list/4","title":"Membrane.Element.Base.Filter.handle_process_list/4","type":"callback","doc":"Callback that is to process buffers. For pads in pull mode it is called when buffers have been demanded (by returning :demand action from any callback). For pads in push mode it is invoked when buffers arrive."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html","title":"Membrane.Element.Base.Mixin.CommonBehaviour","type":"behaviour","doc":"Module defining behaviour common to all elements. When used declares behaviour implementation, provides default callback definitions and imports macros. For more information on implementing elements, see Membrane.Element.Base."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#def_options/1","title":"Membrane.Element.Base.Mixin.CommonBehaviour.def_options/1","type":"macro","doc":"Macro defining options that parametrize element. It automatically generates appropriate struct and documentation. Options are defined by a keyword list, where each key is an option name and is described by another keyword list with following fields: type: atom, used for parsing spec: typespec for value in struct. If ommitted, for types: [:atom, :boolean, :caps, :keyword, :string, :struct, :time] the default typespec is provided, for others typespec is set to any/0 default: default value for option. If not present, value for this option will have to be provided each time options struct is created inspector: function converting fields&#39; value to a string. Used when creating documentation instead of inspect/1 description: string describing an option. It will be used for generating the docs"},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#c:handle_event/4","title":"Membrane.Element.Base.Mixin.CommonBehaviour.handle_event/4","type":"callback","doc":"Callback that is called when event arrives. Events may arrive from both sinks and sources. In filters by default event is forwarded to all sources or sinks, respectively. If event is either Membrane.Event.StartOfStream or Membrane.Event.EndOfStream, notification is sent, to notify the pipeline that data processing is started or finished. This behaviour can be overriden, e.g. by sending end of stream notification after elements internal buffers become empty."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#c:handle_init/1","title":"Membrane.Element.Base.Mixin.CommonBehaviour.handle_init/1","type":"callback","doc":"Callback invoked on initialization of element process. It should parse options and initialize element internal state. Internally it is invoked inside GenServer.init/1 callback."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#c:handle_other/3","title":"Membrane.Element.Base.Mixin.CommonBehaviour.handle_other/3","type":"callback","doc":"Callback invoked when element receives a message that is not recognized as an internal membrane message. Useful for receiving ticks from timer, data sent from NIFs or other stuff."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#c:handle_pad_added/3","title":"Membrane.Element.Base.Mixin.CommonBehaviour.handle_pad_added/3","type":"callback","doc":"Callback that is called when new pad has beed added to element. Executed ONLY for dynamic pads."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#c:handle_pad_removed/3","title":"Membrane.Element.Base.Mixin.CommonBehaviour.handle_pad_removed/3","type":"callback","doc":"Callback that is called when some pad of the element has beed removed. Executed ONLY for dynamic pads."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#c:handle_playing_to_prepared/2","title":"Membrane.Element.Base.Mixin.CommonBehaviour.handle_playing_to_prepared/2","type":"callback","doc":"Callback invoked when element goes to :prepared state from state :playing and should get ready to enter :stopped state. All resources allocated in handle_prepared_to_playing/2 callback should be released here, and no more buffers or demands should be sent."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#c:handle_prepared_to_playing/2","title":"Membrane.Element.Base.Mixin.CommonBehaviour.handle_prepared_to_playing/2","type":"callback","doc":"Callback invoked when element is supposed to start playing (goes from state :prepared to :playing). This is moment when initial demands are sent and first buffers are generated if there are any pads in the push mode."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#c:handle_prepared_to_stopped/2","title":"Membrane.Element.Base.Mixin.CommonBehaviour.handle_prepared_to_stopped/2","type":"callback","doc":"Callback invoked when element is supposed to stop (goes from state :prepared to :stopped). Usually this is the place for releasing all remaining resources used by the element. For example, if element opens a file in handle_stopped_to_prepared/2, this is the place to close it."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#c:handle_shutdown/1","title":"Membrane.Element.Base.Mixin.CommonBehaviour.handle_shutdown/1","type":"callback","doc":"Callback invoked when element is shutting down just before process is exiting. Internally called in GenServer.termintate/2 callback."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#c:handle_stopped_to_prepared/2","title":"Membrane.Element.Base.Mixin.CommonBehaviour.handle_stopped_to_prepared/2","type":"callback","doc":"Callback invoked when element goes to :prepared state from state :stopped and should get ready to enter :playing state. Usually most resources used by the element are allocated here. For example, if element opens a file, this is the place to try to actually open it and return error if that has failed. Such resources should be released in handle_prepared_to_stopped/2."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#c:membrane_element?/0","title":"Membrane.Element.Base.Mixin.CommonBehaviour.membrane_element?/0","type":"callback","doc":"Automatically implemented callback used to determine if module is a membrane element."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#c:membrane_element_type/0","title":"Membrane.Element.Base.Mixin.CommonBehaviour.membrane_element_type/0","type":"callback","doc":"Automatically implemented callback determining whether element is a source, a filter or a sink."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#c:membrane_pads/0","title":"Membrane.Element.Base.Mixin.CommonBehaviour.membrane_pads/0","type":"callback","doc":"Automatically implemented callback returning specification of pads exported by the element. Generated by Membrane.Element.Base.Mixin.SinkBehaviour.def_input_pad/2 and Membrane.Element.Base.Mixin.SourceBehaviour.def_output_pad/2 macros."},{"ref":"Membrane.Element.Base.Mixin.CommonBehaviour.html#t:callback_return_t/0","title":"Membrane.Element.Base.Mixin.CommonBehaviour.callback_return_t/0","type":"type","doc":"Type that defines all valid return values from most callbacks."},{"ref":"Membrane.Element.Base.Mixin.SinkBehaviour.html","title":"Membrane.Element.Base.Mixin.SinkBehaviour","type":"behaviour","doc":"Module defining behaviour for sink and filter elements. When used declares behaviour implementation, provides default callback definitions and imports macros. For more information on implementing elements, see Membrane.Element.Base."},{"ref":"Membrane.Element.Base.Mixin.SinkBehaviour.html#def_input_pad/2","title":"Membrane.Element.Base.Mixin.SinkBehaviour.def_input_pad/2","type":"macro","doc":"Macro that defines input pad for the element. Allows to use one_of/1 and range/2 functions from Membrane.Caps.Matcher without module prefix. It automatically generates documentation from the given definition and adds compile-time caps specs validation. The type Membrane.Element.Pad.input_spec_t/0 describes how the definition of pads should look."},{"ref":"Membrane.Element.Base.Mixin.SinkBehaviour.html#def_input_pads/1","title":"Membrane.Element.Base.Mixin.SinkBehaviour.def_input_pads/1","type":"macro","doc":"Macro that defines multiple input pads for the element. Deprecated in favor of def_input_pad/2"},{"ref":"Membrane.Element.Base.Mixin.SinkBehaviour.html#c:handle_caps/4","title":"Membrane.Element.Base.Mixin.SinkBehaviour.handle_caps/4","type":"callback","doc":"Callback invoked when Element is receiving information about new caps for given pad. In filters those caps are forwarded through all output pads by default."},{"ref":"Membrane.Element.Base.Mixin.SourceBehaviour.html","title":"Membrane.Element.Base.Mixin.SourceBehaviour","type":"behaviour","doc":"Module defining behaviour for source and filter elements. When used declares behaviour implementation, provides default callback definitions and imports macros. For more information on implementing elements, see Membrane.Element.Base."},{"ref":"Membrane.Element.Base.Mixin.SourceBehaviour.html#def_output_pad/2","title":"Membrane.Element.Base.Mixin.SourceBehaviour.def_output_pad/2","type":"macro","doc":"Macro that defines output pad for the element. Allows to use one_of/1 and range/2 functions from Membrane.Caps.Matcher without module prefix. It automatically generates documentation from the given definition and adds compile-time caps specs validation. The type Membrane.Element.Pad.output_spec_t/0 describes how the definition of pads should look."},{"ref":"Membrane.Element.Base.Mixin.SourceBehaviour.html#def_output_pads/1","title":"Membrane.Element.Base.Mixin.SourceBehaviour.def_output_pads/1","type":"macro","doc":"Macro that defines multiple output pads for the element. Deprecated in favor of def_output_pad/2"},{"ref":"Membrane.Element.Base.Mixin.SourceBehaviour.html#c:handle_demand/5","title":"Membrane.Element.Base.Mixin.SourceBehaviour.handle_demand/5","type":"callback","doc":"Callback called when buffers should be emitted by a source or filter. It is called only for output pads in the pull mode, as in their case demand is triggered by the input pad of the subsequent element. In sources, appropriate amount of data should be sent here. In filters, this callback should usually return :demand action with size sufficient for supplying incoming demand. This will result in calling Membrane.Element.Base.Filter.handle_process_list/4, which is to supply the demand. If a source is unable to produce enough buffers, or a filter underestimated returned demand, the :redemand action should be used (see Membrane.Element.Action.redemand_t/0)."},{"ref":"Membrane.Element.Base.Sink.html","title":"Membrane.Element.Base.Sink","type":"behaviour","doc":"Module defining behaviour for sinks - elements consuming data. Behaviours for sinks are specified, besides this place, in modules Membrane.Element.Base.Mixin.CommonBehaviour, and Membrane.Element.Base.Mixin.SinkBehaviour. Sink elements can define only input pads. Job of a usual sink is to receive some data on such pad and consume it (write to a soundcard, send through TCP etc.). If the pad works in pull mode, which is the most common case, then element is also responsible for requesting demands when it is able and willing to consume data (for more details, see Membrane.Element.Action.demand_t/0). Sinks, like all elements, can of course have multiple pads if needed to provide more complex solutions."},{"ref":"Membrane.Element.Base.Sink.html#c:handle_write/4","title":"Membrane.Element.Base.Sink.handle_write/4","type":"callback","doc":"Callback that is called when buffer should be written by the sink. In contrast to handle_write_list/4, it is passed only a single buffer. Called by default implementation of handle_write_list/4."},{"ref":"Membrane.Element.Base.Sink.html#c:handle_write_list/4","title":"Membrane.Element.Base.Sink.handle_write_list/4","type":"callback","doc":"Callback that is called when buffer should be written by the sink. For pads in pull mode it is called when buffers have been demanded (by returning :demand action from any callback). For pads in push mode it is invoked when buffers arrive."},{"ref":"Membrane.Element.Base.Source.html","title":"Membrane.Element.Base.Source","type":"module","doc":"Module that should be used in sources - elements producing data. Declares appropriate behaviours implementation and provides default callbacks implementation. Behaviours for sources are specified in modules Membrane.Element.Base.Mixin.CommonBehaviour and Membrane.Element.Base.Mixin.SourceBehaviour. Source elements can define only output pads. Job of a usual source is to produce some data (read from soundcard, download through HTTP, etc.) and send it through such pad. If the pad works in pull mode, then element is also responsible for receiving demands and send buffers only if they have previously been demanded (for more details, see Membrane.Element.Base.Mixin.SourceBehaviour.handle_demand/5 callback). Sources, like all elements, can of course have multiple pads if needed to provide more complex solutions."},{"ref":"Membrane.Element.LinkError.html","title":"Membrane.Element.LinkError","type":"exception","doc":""},{"ref":"Membrane.Element.Pad.html","title":"Membrane.Element.Pad","type":"module","doc":"Pads are units defined by each element, allowing it to be linked with another elements. This module consists of pads typespecs and utils. Each pad is described by its name, direction, availability, mode and possible caps. For pads to be linkable, these properties have to be compatible. For more information on each of them, check appropriate type in this module. Each link can only consist of exactly two pads."},{"ref":"Membrane.Element.Pad.html#availability_mode/1","title":"Membrane.Element.Pad.availability_mode/1","type":"function","doc":"Returns pad availability mode for given availability."},{"ref":"Membrane.Element.Pad.html#is_availability/1","title":"Membrane.Element.Pad.is_availability/1","type":"macro","doc":""},{"ref":"Membrane.Element.Pad.html#is_availability_dynamic/1","title":"Membrane.Element.Pad.is_availability_dynamic/1","type":"macro","doc":""},{"ref":"Membrane.Element.Pad.html#is_availability_static/1","title":"Membrane.Element.Pad.is_availability_static/1","type":"macro","doc":""},{"ref":"Membrane.Element.Pad.html#is_pad_name/1","title":"Membrane.Element.Pad.is_pad_name/1","type":"macro","doc":""},{"ref":"Membrane.Element.Pad.html#is_pad_ref/1","title":"Membrane.Element.Pad.is_pad_ref/1","type":"macro","doc":""},{"ref":"Membrane.Element.Pad.html#name_by_ref/1","title":"Membrane.Element.Pad.name_by_ref/1","type":"function","doc":"Returns the name for the given pad reference"},{"ref":"Membrane.Element.Pad.html#t:availability_mode_t/0","title":"Membrane.Element.Pad.availability_mode_t/0","type":"type","doc":"Type describing availability mode of a created pad: :static - there always exist exactly one instance of such pad. :dynamic - multiple instances of such pad may be created and removed (which entails executing handle_pad_added and handle_pad_removed callbacks, respectively)."},{"ref":"Membrane.Element.Pad.html#t:availability_t/0","title":"Membrane.Element.Pad.availability_t/0","type":"type","doc":"Values used when defining pad availability: :always - a static pad, which can remain unlinked in stopped state only. :on_request - a dynamic pad, instance of which is created every time it is linked to another pad. Thus linking the pad with k other pads, creates k instances of the pad, and links each with another pad."},{"ref":"Membrane.Element.Pad.html#t:common_spec_options_t/0","title":"Membrane.Element.Pad.common_spec_options_t/0","type":"type","doc":"Pad options used in spec_t/0"},{"ref":"Membrane.Element.Pad.html#t:description_t/0","title":"Membrane.Element.Pad.description_t/0","type":"type","doc":"Type describing a pad. Contains data parsed from spec_t/0"},{"ref":"Membrane.Element.Pad.html#t:direction_t/0","title":"Membrane.Element.Pad.direction_t/0","type":"type","doc":"Defines possible pad directions: :output - data can only be sent through such pad, :input - data can only be received through such pad. One cannot link two pads with the same direction."},{"ref":"Membrane.Element.Pad.html#t:dynamic_id_t/0","title":"Membrane.Element.Pad.dynamic_id_t/0","type":"type","doc":"Possible id of dynamic pad"},{"ref":"Membrane.Element.Pad.html#t:input_spec_t/0","title":"Membrane.Element.Pad.input_spec_t/0","type":"type","doc":"Describes how an input pad should be declared inside an element."},{"ref":"Membrane.Element.Pad.html#t:mode_t/0","title":"Membrane.Element.Pad.mode_t/0","type":"type","doc":"Describes how an element sends and receives data. Modes are strictly related to pad directions: :push output pad - element can send data through such pad whenever it wants. :push input pad - element has to deal with data whenever it comes through such pad, and do it fast enough not to let data accumulate on such pad, what may lead to overflow of element process erlang queue, which is highly unwanted. :pull output pad - element can send data through such pad only if it have already received demand on the pad. Sending small, limited amount of undemanded data is supported and handled by Membrane.Core.InputBuffer. :pull input pad - element receives through such pad only data that it has previously demanded, so that no undemanded data can arrive. Linking pads with different modes is possible, but only in case of output pad working in push mode, and input in pull mode. Moreover, toilet mode of Membrane.Core.InputBuffer has to be enabled then. For more information on transfering data and demands, see docs for element callbacks in Membrane.Element.Base.*."},{"ref":"Membrane.Element.Pad.html#t:name_t/0","title":"Membrane.Element.Pad.name_t/0","type":"type","doc":"Defines the name of pad or group of dynamic pads"},{"ref":"Membrane.Element.Pad.html#t:output_spec_t/0","title":"Membrane.Element.Pad.output_spec_t/0","type":"type","doc":"Describes how an output pad should be declared inside an element."},{"ref":"Membrane.Element.Pad.html#t:ref_t/0","title":"Membrane.Element.Pad.ref_t/0","type":"type","doc":"Defines the term by which the pad instance is identified."},{"ref":"Membrane.Element.Pad.html#t:spec_t/0","title":"Membrane.Element.Pad.spec_t/0","type":"type","doc":"Describes how a pad should be declared in element."},{"ref":"Membrane.Element.Pad.Data.html","title":"Membrane.Element.Pad.Data","type":"module","doc":"Struct describing current pad state. The public fields are: :caps - the most recent Membrane.Caps that have been sent (output) or received (input) on the pad. May be nil if not yet set. :start_of_stream? - flag determining whether Membrane.Event.StartOfStream has been received (or sent) on the pad :end_of_stream? - flag determining whether Membrane.Event.EndOfStream has been received (or sent) on the pad :options - options passed in Membrane.Pipeline.Spec when linking pad Other fields in the struct ARE NOT PART OF THE PUBLIC API and should not be accessed or relied on."},{"ref":"Membrane.Element.Pad.Data.html#t:t/0","title":"Membrane.Element.Pad.Data.t/0","type":"type","doc":""},{"ref":"Membrane.Element.CallbackContext.html","title":"Membrane.Element.CallbackContext","type":"behaviour","doc":"Parent module for all contexts passed to callbacks. The idea of context is to provide access to commonly used information without holding it in elements state. Context differs depending on callback."},{"ref":"Membrane.Element.CallbackContext.html#c:from_state/2","title":"Membrane.Element.CallbackContext.from_state/2","type":"macrocallback","doc":""},{"ref":"Membrane.Element.CallbackContext.Caps.html","title":"Membrane.Element.CallbackContext.Caps","type":"module","doc":"Structure representing a context that is passed to the element when receiving information about new caps for given pad. The old_caps field contains caps previously present on the pad, and is equal to pads[pad].caps field."},{"ref":"Membrane.Element.CallbackContext.Caps.html#t:t/0","title":"Membrane.Element.CallbackContext.Caps.t/0","type":"type","doc":""},{"ref":"Membrane.Element.CallbackContext.Demand.html","title":"Membrane.Element.CallbackContext.Demand","type":"module","doc":"Structure representing a context that is passed to the element when processing incoming demand."},{"ref":"Membrane.Element.CallbackContext.Demand.html#t:t/0","title":"Membrane.Element.CallbackContext.Demand.t/0","type":"type","doc":""},{"ref":"Membrane.Element.CallbackContext.Event.html","title":"Membrane.Element.CallbackContext.Event","type":"module","doc":"Structure representing a context that is passed to the element when handling event."},{"ref":"Membrane.Element.CallbackContext.Event.html#t:t/0","title":"Membrane.Element.CallbackContext.Event.t/0","type":"type","doc":""},{"ref":"Membrane.Element.CallbackContext.Other.html","title":"Membrane.Element.CallbackContext.Other","type":"module","doc":"Structure representing a context that is passed to the callback when element receives unrecognized message."},{"ref":"Membrane.Element.CallbackContext.Other.html#t:t/0","title":"Membrane.Element.CallbackContext.Other.t/0","type":"type","doc":""},{"ref":"Membrane.Element.CallbackContext.PadAdded.html","title":"Membrane.Element.CallbackContext.PadAdded","type":"module","doc":"Structure representing a context that is passed to the element when when new pad added is created"},{"ref":"Membrane.Element.CallbackContext.PadAdded.html#t:t/0","title":"Membrane.Element.CallbackContext.PadAdded.t/0","type":"type","doc":""},{"ref":"Membrane.Element.CallbackContext.PadRemoved.html","title":"Membrane.Element.CallbackContext.PadRemoved","type":"module","doc":"Structure representing a context that is passed to the element when when new pad added is created"},{"ref":"Membrane.Element.CallbackContext.PadRemoved.html#t:t/0","title":"Membrane.Element.CallbackContext.PadRemoved.t/0","type":"type","doc":""},{"ref":"Membrane.Element.CallbackContext.PlaybackChange.html","title":"Membrane.Element.CallbackContext.PlaybackChange","type":"module","doc":"Structure representing a context that is passed to the callback of the element when it changes playback state"},{"ref":"Membrane.Element.CallbackContext.PlaybackChange.html#t:t/0","title":"Membrane.Element.CallbackContext.PlaybackChange.t/0","type":"type","doc":""},{"ref":"Membrane.Element.CallbackContext.Process.html","title":"Membrane.Element.CallbackContext.Process","type":"module","doc":"Structure representing a context that is passed to the element when new buffer arrives."},{"ref":"Membrane.Element.CallbackContext.Process.html#t:t/0","title":"Membrane.Element.CallbackContext.Process.t/0","type":"type","doc":""},{"ref":"Membrane.Element.CallbackContext.Write.html","title":"Membrane.Element.CallbackContext.Write","type":"module","doc":"Structure representing a context that is passed to the element when new buffer arrives to the sink."},{"ref":"Membrane.Element.CallbackContext.Write.html#t:t/0","title":"Membrane.Element.CallbackContext.Write.t/0","type":"type","doc":""},{"ref":"Membrane.Buffer.html","title":"Membrane.Buffer","type":"module","doc":"Structure representing a single chunk of data that flows between elements. For now, it is just a wrapper around bitstring with optionally some metadata attached to it, but in future releases we plan to support different payload types."},{"ref":"Membrane.Buffer.html#print/1","title":"Membrane.Buffer.print/1","type":"function","doc":"Converts buffer/buffers to the format in which they can be logged with Membrane.Log."},{"ref":"Membrane.Buffer.html#t:metadata_t/0","title":"Membrane.Buffer.metadata_t/0","type":"type","doc":""},{"ref":"Membrane.Buffer.html#t:t/0","title":"Membrane.Buffer.t/0","type":"type","doc":""},{"ref":"Membrane.Buffer.Metric.html","title":"Membrane.Buffer.Metric","type":"behaviour","doc":"Specifies API for metrics that analyze data in terms of a given unit"},{"ref":"Membrane.Buffer.Metric.html#c:buffers_size/1","title":"Membrane.Buffer.Metric.buffers_size/1","type":"callback","doc":""},{"ref":"Membrane.Buffer.Metric.html#from_unit/1","title":"Membrane.Buffer.Metric.from_unit/1","type":"function","doc":""},{"ref":"Membrane.Buffer.Metric.html#c:pullbuffer_preferred_size/0","title":"Membrane.Buffer.Metric.pullbuffer_preferred_size/0","type":"callback","doc":""},{"ref":"Membrane.Buffer.Metric.html#c:split_buffers/2","title":"Membrane.Buffer.Metric.split_buffers/2","type":"callback","doc":""},{"ref":"Membrane.Buffer.Metric.html#t:unit_t/0","title":"Membrane.Buffer.Metric.unit_t/0","type":"type","doc":""},{"ref":"Membrane.Buffer.Metric.ByteSize.html","title":"Membrane.Buffer.Metric.ByteSize","type":"module","doc":"Implementation of Membrane.Buffer.Metric for the :bytes unit"},{"ref":"Membrane.Buffer.Metric.ByteSize.html#buffers_size/1","title":"Membrane.Buffer.Metric.ByteSize.buffers_size/1","type":"function","doc":"Callback implementation for Membrane.Buffer.Metric.buffers_size/1."},{"ref":"Membrane.Buffer.Metric.ByteSize.html#pullbuffer_preferred_size/0","title":"Membrane.Buffer.Metric.ByteSize.pullbuffer_preferred_size/0","type":"function","doc":"Callback implementation for Membrane.Buffer.Metric.pullbuffer_preferred_size/0."},{"ref":"Membrane.Buffer.Metric.ByteSize.html#split_buffers/2","title":"Membrane.Buffer.Metric.ByteSize.split_buffers/2","type":"function","doc":"Callback implementation for Membrane.Buffer.Metric.split_buffers/2."},{"ref":"Membrane.Buffer.Metric.Count.html","title":"Membrane.Buffer.Metric.Count","type":"module","doc":"Implementation of Membrane.Buffer.Metric for the :buffers unit"},{"ref":"Membrane.Buffer.Metric.Count.html#buffers_size/1","title":"Membrane.Buffer.Metric.Count.buffers_size/1","type":"function","doc":"Callback implementation for Membrane.Buffer.Metric.buffers_size/1."},{"ref":"Membrane.Buffer.Metric.Count.html#pullbuffer_preferred_size/0","title":"Membrane.Buffer.Metric.Count.pullbuffer_preferred_size/0","type":"function","doc":"Callback implementation for Membrane.Buffer.Metric.pullbuffer_preferred_size/0."},{"ref":"Membrane.Buffer.Metric.Count.html#split_buffers/2","title":"Membrane.Buffer.Metric.Count.split_buffers/2","type":"function","doc":"Callback implementation for Membrane.Buffer.Metric.split_buffers/2."},{"ref":"Membrane.Caps.html","title":"Membrane.Caps","type":"module","doc":"Describes capabilities of some pad. Every pad has some capabilities, which define a type of data that pad is expecting. This format can be, for example, raw audio with specific sample rate or encoded audio in given format. To link two pads together, their capabilities have to be compatible."},{"ref":"Membrane.Caps.html#t:t/0","title":"Membrane.Caps.t/0","type":"type","doc":""},{"ref":"Membrane.Caps.Matcher.html","title":"Membrane.Caps.Matcher","type":"module","doc":"Module that allows to specify valid caps and verify that they match specification. Caps specifications (specs) should be in one of the formats: simply module name of the desired caps (e.g. Membrane.Caps.Audio.Raw or Raw with proper alias) tuple with module name and keyword list of specs for specific caps fields (e.g. {Raw, format: :s24le}) list of the formats described above Field values can be specified in following ways: By a raw value for the field (e.g. :s24le) Using range/2 for values comparable with Kernel.&lt;=/2 and Kernel.&gt;=/2 (e.g. Matcher.range(0, 255)) With one_of/1 and a list of valid values (e.g Matcher.one_of([:u8, :s16le, :s32le])) Checks on the values from list are performed recursively i.e. it can contain another range/2, for example Matcher.one_of([0, Matcher.range(2, 4), Matcher.range(10, 20)]) If the specs are defined inside of Membrane.Element.Base.Mixin.SinkBehaviour.def_input_pad/2 and Membrane.Element.Base.Mixin.SourceBehaviour.def_output_pad/2 module name can be omitted from range/2 and one_of/1 calls. Example Below is a pad definition with an example of specs for caps matcher: alias Membrane.Caps.Video.Raw def_input_pad :input, demand_unit: :buffers, caps: {Raw, format: one_of([:I420, :I422]), aligned: true}"},{"ref":"Membrane.Caps.Matcher.html#match?/2","title":"Membrane.Caps.Matcher.match?/2","type":"function","doc":"Function determining whether the caps match provided specs. When :any is used as specs, caps can by anything (i.e. they can be invalid)"},{"ref":"Membrane.Caps.Matcher.html#one_of/1","title":"Membrane.Caps.Matcher.one_of/1","type":"function","doc":"Returns opaque specification of list of valid values for caps field."},{"ref":"Membrane.Caps.Matcher.html#range/2","title":"Membrane.Caps.Matcher.range/2","type":"function","doc":"Returns opaque specification of range of valid values for caps field."},{"ref":"Membrane.Caps.Matcher.html#validate_specs/1","title":"Membrane.Caps.Matcher.validate_specs/1","type":"function","doc":"Function used to make sure caps specs are valid. In particular, valid caps: Have shape described by caps_specs_t/0 type If they contain keyword list, the keys are present in requested caps type It returns :ok when caps are valid and {:error, reason} otherwise"},{"ref":"Membrane.Caps.Matcher.html#t:caps_spec_t/0","title":"Membrane.Caps.Matcher.caps_spec_t/0","type":"type","doc":""},{"ref":"Membrane.Caps.Matcher.html#t:caps_specs_t/0","title":"Membrane.Caps.Matcher.caps_specs_t/0","type":"type","doc":""},{"ref":"Membrane.Caps.Matcher.html#t:one_of_t/0","title":"Membrane.Caps.Matcher.one_of_t/0","type":"opaque","doc":""},{"ref":"Membrane.Caps.Matcher.html#t:range_t/0","title":"Membrane.Caps.Matcher.range_t/0","type":"opaque","doc":""},{"ref":"Membrane.Event.html","title":"Membrane.Event","type":"module","doc":"Event is an entity that can be sent between elements. Events can flow either downstream or upstream - they can be sent with Membrane.Element.Action.event_t/0, and can be handled in Membrane.Element.Base.Mixin.CommonBehaviour.handle_event/4. Each event is to implement Membrane.EventProtocol, which allows to configure its behaviour."},{"ref":"Membrane.Event.html#async?/1","title":"Membrane.Event.async?/1","type":"function","doc":"See Membrane.EventProtocol.async?/1."},{"ref":"Membrane.Event.html#event?/1","title":"Membrane.Event.event?/1","type":"function","doc":""},{"ref":"Membrane.Event.html#sticky?/1","title":"Membrane.Event.sticky?/1","type":"function","doc":"See Membrane.EventProtocol.sticky?/1."},{"ref":"Membrane.Event.html#t:t/0","title":"Membrane.Event.t/0","type":"type","doc":""},{"ref":"Membrane.Event.Discontinuity.html","title":"Membrane.Event.Discontinuity","type":"module","doc":"Generic discontinuity event. This event means that flow of buffers in the stream was interrupted, but stream itself is not done. Frequent reasons for this are soundcards&#39; drops while capturing sound, network data loss etc. If duration of the discontinuity is known, it can be passed as an argument."},{"ref":"Membrane.Event.Discontinuity.html#t:duration_t/0","title":"Membrane.Event.Discontinuity.duration_t/0","type":"type","doc":""},{"ref":"Membrane.Event.Discontinuity.html#t:t/0","title":"Membrane.Event.Discontinuity.t/0","type":"type","doc":""},{"ref":"Membrane.Event.EndOfStream.html","title":"Membrane.Event.EndOfStream","type":"module","doc":"Generic End of Stream event. This event means that all buffers from the stream were processed and no further buffers are expected to arrive. Membrane.Element.Base.Mixin.CommonBehaviour.handle_event/4 by default sends notification to pipeline when this event is handled."},{"ref":"Membrane.Event.EndOfStream.html#t:t/0","title":"Membrane.Event.EndOfStream.t/0","type":"type","doc":""},{"ref":"Membrane.Event.StartOfStream.html","title":"Membrane.Event.StartOfStream","type":"module","doc":"Generic Start of Stream event. This event means that first buffers have been sent and will arrive to the pad right away after handling the event. Membrane.Element.Base.Mixin.CommonBehaviour.handle_event/4 by default sends notification to pipeline when this event is handled."},{"ref":"Membrane.Event.StartOfStream.html#t:t/0","title":"Membrane.Event.StartOfStream.t/0","type":"type","doc":""},{"ref":"Membrane.Event.Underrun.html","title":"Membrane.Event.Underrun","type":"module","doc":"Generic underrun event. This event means that certain element is willing to consume more buffers, but there are none available. It makes sense to use this event as an upstream event to notify previous elements in the pipeline that they should generate more buffers."},{"ref":"Membrane.Event.Underrun.html#t:t/0","title":"Membrane.Event.Underrun.t/0","type":"type","doc":""},{"ref":"Membrane.EventProtocol.html","title":"Membrane.EventProtocol","type":"protocol","doc":"Protocol that allows to configure behaviour of Membrane.Events. Each event has to implement or derive this protocol."},{"ref":"Membrane.EventProtocol.html#async?/1","title":"Membrane.EventProtocol.async?/1","type":"function","doc":"Determines whether event is synchronized with buffers (sync) or not (async). Defaults to false (sync). Buffers and sync events are always received in the same order they are sent. Async events are handled before any enqueued buffers that are waiting to be processed (e.g. in Membrane.InputBuffer)."},{"ref":"Membrane.EventProtocol.html#sticky?/1","title":"Membrane.EventProtocol.sticky?/1","type":"function","doc":"Specifies whether event is sent right away (not sticky), or it is &#39;pushed&#39; by the next sent buffer (sticky). Defaults to false (not sticky). Returning a sticky event from a callback stores it in a queue. When the next buffer is to be sent, all events from the queue are sent before it. An example can be the Membrane.Event.StartOfStream event."},{"ref":"Membrane.EventProtocol.html#t:t/0","title":"Membrane.EventProtocol.t/0","type":"type","doc":""},{"ref":"Membrane.EventProtocol.DefaultImpl.html","title":"Membrane.EventProtocol.DefaultImpl","type":"module","doc":"Default implementation of Membrane.EventProtocol. If used in defimpl, not implemented callbacks fallback to default ones."},{"ref":"Membrane.Notification.html","title":"Membrane.Notification","type":"module","doc":"A notification is a message sent from Membrane.Element to a Membrane.Pipeline via action Membrane.Element.Action.notify_t/0 returned from any callback. A notification can be handled in Pipeline with callback Membrane.Pipeline.handle_notification/3 The example of a notification is tuple {:end_of_stream, pad_ref} that, by default, is sent when handling Membrane.Event.EndOfStream event"},{"ref":"Membrane.Notification.html#t:t/0","title":"Membrane.Notification.t/0","type":"type","doc":""},{"ref":"Membrane.Payload.html","title":"Membrane.Payload","type":"protocol","doc":"This protocol describes actions common to all payload types. The most basic payload type is simply a binary for which Elixir.Membrane.Payload is implemented by the Membrane Core."},{"ref":"Membrane.Payload.html#concat/2","title":"Membrane.Payload.concat/2","type":"function","doc":"Concatenates the contents of two payloads."},{"ref":"Membrane.Payload.html#drop/2","title":"Membrane.Payload.drop/2","type":"function","doc":"Drops first n bytes of payload."},{"ref":"Membrane.Payload.html#module/1","title":"Membrane.Payload.module/1","type":"function","doc":"Returns a module responsible for this type of payload and implementing Membrane.Payload.Behaviour"},{"ref":"Membrane.Payload.html#size/1","title":"Membrane.Payload.size/1","type":"function","doc":"Returns total size of payload in bytes"},{"ref":"Membrane.Payload.html#split_at/2","title":"Membrane.Payload.split_at/2","type":"function","doc":"Splits the payload at given position (1st part has the size equal to at_pos argument) at_pos has to be greater than 0 and smaller than the size of payload, otherwise an error is raised. This guarantees returned payloads are never empty."},{"ref":"Membrane.Payload.html#to_binary/1","title":"Membrane.Payload.to_binary/1","type":"function","doc":"Converts payload into binary"},{"ref":"Membrane.Payload.html#t:t/0","title":"Membrane.Payload.t/0","type":"type","doc":""},{"ref":"Membrane.Payload.Behaviour.html","title":"Membrane.Payload.Behaviour","type":"behaviour","doc":"Behaviour that should be implemented by every module that has Membrane.Payload protocol implementation."},{"ref":"Membrane.Payload.Behaviour.html#c:empty/0","title":"Membrane.Payload.Behaviour.empty/0","type":"callback","doc":"Creates an empty payload"},{"ref":"Membrane.Payload.Behaviour.html#c:new/1","title":"Membrane.Payload.Behaviour.new/1","type":"callback","doc":"Creates a new payload initialized with the given binary"},{"ref":"Membrane.Payload.Binary.html","title":"Membrane.Payload.Binary","type":"module","doc":"Membrane.Payload.Behaviour implementation for binary payload. Complements Membrane.Payload protocol implementation."},{"ref":"Membrane.Log.html","title":"Membrane.Log","type":"module","doc":"Mixin for logging using simple functions such as info/1, debug/1 in other modules."},{"ref":"Membrane.Log.html#debug/2","title":"Membrane.Log.debug/2","type":"macro","doc":""},{"ref":"Membrane.Log.html#info/2","title":"Membrane.Log.info/2","type":"macro","doc":""},{"ref":"Membrane.Log.html#log/3","title":"Membrane.Log.log/3","type":"macro","doc":""},{"ref":"Membrane.Log.html#or_warn_error/3","title":"Membrane.Log.or_warn_error/3","type":"macro","doc":""},{"ref":"Membrane.Log.html#warn/2","title":"Membrane.Log.warn/2","type":"macro","doc":""},{"ref":"Membrane.Log.html#warn_error/3","title":"Membrane.Log.warn_error/3","type":"macro","doc":""},{"ref":"Membrane.Log.html#t:level_t/0","title":"Membrane.Log.level_t/0","type":"type","doc":""},{"ref":"Membrane.Log.Logger.html","title":"Membrane.Log.Logger","type":"module","doc":"Module containing functions spawning, shutting down, and handling messages sent to logger."},{"ref":"Membrane.Log.Logger.html#child_spec/1","title":"Membrane.Log.Logger.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Membrane.Log.Logger.html#shutdown/2","title":"Membrane.Log.Logger.shutdown/2","type":"function","doc":"Stops given logger process. It will wait for reply for amount of time passed as second argument (in milliseconds). Will trigger calling handle_shutdown/2 logger callback. Returns :ok."},{"ref":"Membrane.Log.Logger.html#start/3","title":"Membrane.Log.Logger.start/3","type":"function","doc":"Starts process for logger of given module, initialized with given options outside of the supervision tree. Works similarly to GenServer.start/3 and has the same return values."},{"ref":"Membrane.Log.Logger.html#start_link/3","title":"Membrane.Log.Logger.start_link/3","type":"function","doc":"Starts process for logger of given module, initialized with given options and links it to the current process in the supervision tree. Works similarly to GenServer.start_link/3 and has the same return values."},{"ref":"Membrane.Log.Logger.html#t:logger_options_t/0","title":"Membrane.Log.Logger.logger_options_t/0","type":"type","doc":""},{"ref":"Membrane.Log.Logger.html#t:message_t/0","title":"Membrane.Log.Logger.message_t/0","type":"type","doc":""},{"ref":"Membrane.Log.Logger.html#t:msg_level_t/0","title":"Membrane.Log.Logger.msg_level_t/0","type":"type","doc":""},{"ref":"Membrane.Log.Logger.html#t:on_start/0","title":"Membrane.Log.Logger.on_start/0","type":"type","doc":""},{"ref":"Membrane.Log.Logger.html#t:process_options_t/0","title":"Membrane.Log.Logger.process_options_t/0","type":"type","doc":""},{"ref":"Membrane.Log.Logger.html#t:tag_t/0","title":"Membrane.Log.Logger.tag_t/0","type":"type","doc":""},{"ref":"Membrane.Log.Logger.Base.html","title":"Membrane.Log.Logger.Base","type":"behaviour","doc":"This is a base module used by all logger implementations."},{"ref":"Membrane.Log.Logger.Base.html#c:handle_init/1","title":"Membrane.Log.Logger.Base.handle_init/1","type":"callback","doc":"Callback invoked when logger is initialized, right after new process is spawned. On success it should return {:ok, initial_logger_state}."},{"ref":"Membrane.Log.Logger.Base.html#c:handle_log/5","title":"Membrane.Log.Logger.Base.handle_log/5","type":"callback","doc":"Callback invoked when new log message is received. Callback delivers 5 arguments: atom containing log level message - in IO list format time tags (list of atoms, e.g. module name) internal logger state On success, it returns {:ok, new_state}. it will just update logger&#39;s state to the new state. If it returns {:error, reason, new_state} it indicates that something went wrong, and logger was unable to handle log. State will be updated to the new state."},{"ref":"Membrane.Log.Logger.Base.html#c:handle_shutdown/1","title":"Membrane.Log.Logger.Base.handle_shutdown/1","type":"callback","doc":"Callback invoked when logger is shutting down just before process is exiting. It will receive the logger state. Return value is ignored."},{"ref":"Membrane.Log.Router.html","title":"Membrane.Log.Router","type":"module","doc":"Defines a router that dispatches logs to instances of Membrane.Log.Logger.Base"},{"ref":"Membrane.Log.Router.html#child_spec/1","title":"Membrane.Log.Router.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Membrane.Log.Router.html#level_to_val/1","title":"Membrane.Log.Router.level_to_val/1","type":"function","doc":"Converts atom with level to its number representation Valid atoms are: :debug :info :warn"},{"ref":"Membrane.Log.Router.html#send_log/4","title":"Membrane.Log.Router.send_log/4","type":"function","doc":"Sends asynchronous call to the router, requesting it to forward log message to appropriate loggers. This functions assumes that passed log has level equal or greater than global level."},{"ref":"Membrane.Log.Router.html#start_link/1","title":"Membrane.Log.Router.start_link/1","type":"function","doc":"Starts router as a separate process. Options are passed to Supervisor.start_link/3."},{"ref":"Membrane.Log.Supervisor.html","title":"Membrane.Log.Supervisor","type":"module","doc":"Module responsible for supervising router_level loggers. It is also responsible for receiving and routing log messages to appropriate loggers. It is spawned upon application boot."},{"ref":"Membrane.Log.Supervisor.html#add_logger/3","title":"Membrane.Log.Supervisor.add_logger/3","type":"function","doc":"Initializes logger and adds it to the supervision tree. As arguments, it expects module name, logger options and process/logger id If successful returns :ok On error returns :invalid_module"},{"ref":"Membrane.Log.Supervisor.html#child_spec/1","title":"Membrane.Log.Supervisor.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Membrane.Log.Supervisor.html#each_logger/1","title":"Membrane.Log.Supervisor.each_logger/1","type":"function","doc":"Iterates through list of children and executes given function on every child. Should return :ok."},{"ref":"Membrane.Log.Supervisor.html#remove_logger/1","title":"Membrane.Log.Supervisor.remove_logger/1","type":"function","doc":"Removes logger from the supervision tree If successful returns :ok If logger could not be found, returns corresponding error"},{"ref":"Membrane.Log.Supervisor.html#start_link/2","title":"Membrane.Log.Supervisor.start_link/2","type":"function","doc":"Starts the Supervisor. Options are passed to Supervisor.start_link/3."},{"ref":"Membrane.Log.Supervisor.html#t:child_id_t/0","title":"Membrane.Log.Supervisor.child_id_t/0","type":"type","doc":""},{"ref":"Membrane.Testing.DataSource.html","title":"Membrane.Testing.DataSource","type":"module","doc":"Testing Element for supplying data from list passed through options. Element options Passed via struct Membrane.Testing.DataSource.t/0 data Required List of payloads that will be sent through :output pad. Pads :output Availability:always Caps:any Direction:output Mode:pull"},{"ref":"Membrane.Testing.DataSource.html#membrane_pads/0","title":"Membrane.Testing.DataSource.membrane_pads/0","type":"function","doc":"Returns pads descriptions for Membrane.Testing.DataSource"},{"ref":"Membrane.Testing.DataSource.html#options/0","title":"Membrane.Testing.DataSource.options/0","type":"function","doc":"Returns description of options available for this module"},{"ref":"Membrane.Testing.DataSource.html#t:t/0","title":"Membrane.Testing.DataSource.t/0","type":"type","doc":"Struct containing options for Membrane.Testing.DataSource"},{"ref":"Membrane.Testing.Event.html","title":"Membrane.Testing.Event","type":"module","doc":"Empty event that can be used in tests"},{"ref":"Membrane.Testing.Pipeline.html","title":"Membrane.Testing.Pipeline","type":"module","doc":"This Pipeline was created to reduce testing boilerplate and ease communication with it&#39;s elements. It also provides utility for receiving messages when Pipeline playback state changes and notifications it receives. Starting Pipeline When you want a build Pipeline to test your elements you need three things: Pipeline Module List of elements Links between those elements When creating pipelines for tests the only essential part is the list of elements. In most cases during the tests elements are linked in a way that :output pad is linked to :input pad of subsequent element. So we only need to pass a list of elements and links can be generated automatically. To start a testing pipeline you need to build Membrane.Testing.Pipeline.Options struct and pass to to Membrane.Testing.Pipeline.start_link/2. options = %Membrane.Testing.Pipeline.Options { elements: [ el1: MembraneElement1, el2: MembraneElement2, ... ] } {:ok, pipeline} = Membrane.Testing.Pipeline.start_link(options) See Membrane.Testing.Pipeline.Options for available options. Links are generated by populate_links/1. Receiving notifications about callbacks invocation In some cases, you want to get a notification when Pipeline reaches a certain state. You can achieve that by monitoring pipeline callbacks. options = %Membrane.Testing.Pipeline.Options { monitored_callbacks: [:handle_prepared_to_playing], test_process: pid ... } First, you need to configure which callbacks are to be monitored by putting their names into monitored_callbacks field of Options struct. Check Membrane.Testing.Pipeline.Options.pipeline_callback/0 for list of available callback names. You also need to pass a PID of the process that will receive messages. import Membrane.Testing.Pipeline.Assertions assert_receive_message :handle_prepared_to_playing Then it is a matter of waiting for the right message to come. You can do that easily by using Membrane.Testing.Pipeline.Assertions.assert_receive_message/3. Messaging children You can send messages to children using their names specified in the elements list. Please check message_child/3 for more details."},{"ref":"Membrane.Testing.Pipeline.html#message_child/3","title":"Membrane.Testing.Pipeline.message_child/3","type":"function","doc":"Sends message to a child by Element name. Examples Knowing that pipeline has child named sink, message can be sent as follows: message_child(pipeline, :sink, {:message, &quot;to handle&quot;})"},{"ref":"Membrane.Testing.Pipeline.html#play/1","title":"Membrane.Testing.Pipeline.play/1","type":"function","doc":"Changes playback state of pipeline to :playing A proxy for Membrane.Pipeline.play/1"},{"ref":"Membrane.Testing.Pipeline.html#populate_links/1","title":"Membrane.Testing.Pipeline.populate_links/1","type":"function","doc":"Links subsequent elements using default pads (linking :input to :output of previous element). Examples iex&gt; Pipeline.populate_links([el1: MembraneElement1, el2: MembraneElement2]) %{{:el1, :output} =&gt; {:el2, :input}}"},{"ref":"Membrane.Testing.Pipeline.html#prepare/1","title":"Membrane.Testing.Pipeline.prepare/1","type":"function","doc":"Changes playback state to :prepared. A proxy for Membrane.Pipeline.prepare/1"},{"ref":"Membrane.Testing.Pipeline.html#start/2","title":"Membrane.Testing.Pipeline.start/2","type":"function","doc":"Starts the pipeline Membrane.Testing.Pipeline without linking it to the current process. A proxy for Membrane.Pipeline.start/3"},{"ref":"Membrane.Testing.Pipeline.html#start_link/2","title":"Membrane.Testing.Pipeline.start_link/2","type":"function","doc":"Starts the pipeline Membrane.Testing.Pipeline and links it to the current process. A proxy for Membrane.Pipeline.start_link/3"},{"ref":"Membrane.Testing.Pipeline.html#stop/1","title":"Membrane.Testing.Pipeline.stop/1","type":"function","doc":"Changes playback state to :stopped. A proxy for Membrane.Pipeline.stop/1"},{"ref":"Membrane.Testing.Pipeline.html#stop_and_terminate/1","title":"Membrane.Testing.Pipeline.stop_and_terminate/1","type":"function","doc":"Changes pipeline&#39;s playback state to :stopped and terminates its process. A proxy for Membrane.Pipeline.stop_and_terminate/1"},{"ref":"Membrane.Testing.Pipeline.Assertions.html","title":"Membrane.Testing.Pipeline.Assertions","type":"module","doc":"Assertions that can be used with Membrane.Testing.Pipeline in tests"},{"ref":"Membrane.Testing.Pipeline.Assertions.html#assert_receive_message/3","title":"Membrane.Testing.Pipeline.Assertions.assert_receive_message/3","type":"macro","doc":"Asserts that a message sent from Membrane.Testing.Pipeline matching pattern was or is going to be received within the timeout period, specified in milliseconds. The pattern argument must be a match pattern. Flunks with failure_message if a message matching pattern is not received. For example to wait for message indicating handle_prepared_to_playing callback was called: assert_receive_message :handle_prepared_to_playing"},{"ref":"Membrane.Testing.Pipeline.Options.html","title":"Membrane.Testing.Pipeline.Options","type":"module","doc":"Structure representing options passed to testing pipeline. Monitored Callbacks A message will be sent to test process if the invoked callback is in the list of monitored callbacks. See pipeline_callback/0 for available callbacks. Test Process pid of process that shall receive messages when Pipeline invokes playback state change callback and receives notification. Elements List of element specs. Links Map describing links between elements. If links are not present or set to nil they will be populated automatically based on elements order using default pad names."},{"ref":"Membrane.Testing.Pipeline.Options.html#t:pipeline_callback/0","title":"Membrane.Testing.Pipeline.Options.pipeline_callback/0","type":"type","doc":"Defines supported callback names."},{"ref":"Membrane.Testing.Pipeline.Options.html#t:t/0","title":"Membrane.Testing.Pipeline.Options.t/0","type":"type","doc":""},{"ref":"Membrane.Testing.Sink.html","title":"Membrane.Testing.Sink","type":"module","doc":"Sink Element that will send every buffer it receives to the pid passed as argument. Element options Passed via struct Membrane.Testing.Sink.t/0 autodemand Default value: true If true element will automatically make demands. If it is set to false demand has to be triggered manually by sending :make_demand message. target Required PID of process that will receive incoming buffers. Pads :input Availability:always Caps:any Demand&nbsp;unit:buffers Direction:input Mode:pull"},{"ref":"Membrane.Testing.Sink.html#membrane_pads/0","title":"Membrane.Testing.Sink.membrane_pads/0","type":"function","doc":"Returns pads descriptions for Membrane.Testing.Sink"},{"ref":"Membrane.Testing.Sink.html#options/0","title":"Membrane.Testing.Sink.options/0","type":"function","doc":"Returns description of options available for this module"},{"ref":"Membrane.Testing.Sink.html#t:t/0","title":"Membrane.Testing.Sink.t/0","type":"type","doc":"Struct containing options for Membrane.Testing.Sink"},{"ref":"Membrane.Testing.Source.html","title":"Membrane.Testing.Source","type":"module","doc":"Testing Element for supplying data based on generator function passed through options. Element options Passed via struct Membrane.Testing.Source.t/0 actions_generator Default value: &amp;Membrane.Testing.Source.default_buf_gen/2 Function invoked each time handle_demand is invoked. It is an action generator that takes two arguments. First argument is counter which is incremented by 1 every call and second argument represents size of demand. Pads :output Availability:always Caps:any Direction:output Mode:pull"},{"ref":"Membrane.Testing.Source.html#default_buf_gen/2","title":"Membrane.Testing.Source.default_buf_gen/2","type":"function","doc":""},{"ref":"Membrane.Testing.Source.html#membrane_pads/0","title":"Membrane.Testing.Source.membrane_pads/0","type":"function","doc":"Returns pads descriptions for Membrane.Testing.Source"},{"ref":"Membrane.Testing.Source.html#options/0","title":"Membrane.Testing.Source.options/0","type":"function","doc":"Returns description of options available for this module"},{"ref":"Membrane.Testing.Source.html#t:t/0","title":"Membrane.Testing.Source.t/0","type":"type","doc":"Struct containing options for Membrane.Testing.Source"},{"ref":"Membrane.Time.html","title":"Membrane.Time","type":"module","doc":"Module containing functions needed to perform handling of time. Membrane always internally uses nanosecond as a time unit. This is how all time units should represented in the code unless there&#39;s a good reason to act differently. Please note that Erlang VM may internally use different units and that may differ from platform to platform. Still, unless you need to perform calculations that do not touch hardware clock, you should use Membrane units for consistency."},{"ref":"Membrane.Time.html#day/1","title":"Membrane.Time.day/1","type":"function","doc":"Returns given days in internal Membrane time units. Inlined by the compiler."},{"ref":"Membrane.Time.html#days/1","title":"Membrane.Time.days/1","type":"function","doc":"The same as day/1. Inlined by the compiler."},{"ref":"Membrane.Time.html#from_datetime/1","title":"Membrane.Time.from_datetime/1","type":"function","doc":"Converts DateTime to internal Membrane time units. Inlined by the compiler."},{"ref":"Membrane.Time.html#from_iso8601!/1","title":"Membrane.Time.from_iso8601!/1","type":"function","doc":"Converts iso8601 string to internal Membrane time units. If value is invalid, throws match error. Inlined by the compiler."},{"ref":"Membrane.Time.html#hour/1","title":"Membrane.Time.hour/1","type":"function","doc":"Returns given hours in internal Membrane time units. Inlined by the compiler."},{"ref":"Membrane.Time.html#hours/1","title":"Membrane.Time.hours/1","type":"function","doc":"The same as hour/1. Inlined by the compiler."},{"ref":"Membrane.Time.html#is_native_t/1","title":"Membrane.Time.is_native_t/1","type":"macro","doc":"Checks whether value is Membrane.Time.native_t"},{"ref":"Membrane.Time.html#is_t/1","title":"Membrane.Time.is_t/1","type":"macro","doc":"Checks whether value is Membrane.Time.t"},{"ref":"Membrane.Time.html#microsecond/1","title":"Membrane.Time.microsecond/1","type":"function","doc":"Returns given microseconds in internal Membrane time units. Inlined by the compiler."},{"ref":"Membrane.Time.html#microseconds/1","title":"Membrane.Time.microseconds/1","type":"function","doc":"The same as microsecond/1. Inlined by the compiler."},{"ref":"Membrane.Time.html#millisecond/1","title":"Membrane.Time.millisecond/1","type":"function","doc":"Returns given milliseconds in internal Membrane time units. Inlined by the compiler."},{"ref":"Membrane.Time.html#milliseconds/1","title":"Membrane.Time.milliseconds/1","type":"function","doc":"The same as millisecond/1. Inlined by the compiler."},{"ref":"Membrane.Time.html#minute/1","title":"Membrane.Time.minute/1","type":"function","doc":"Returns given minutes in internal Membrane time units. Inlined by the compiler."},{"ref":"Membrane.Time.html#minutes/1","title":"Membrane.Time.minutes/1","type":"function","doc":"The same as minute/1. Inlined by the compiler."},{"ref":"Membrane.Time.html#monotonic_time/0","title":"Membrane.Time.monotonic_time/0","type":"function","doc":"Returns current monotonic time based on System.monotonic_time/0 in internal Membrane time units. Inlined by the compiler."},{"ref":"Membrane.Time.html#nanosecond/1","title":"Membrane.Time.nanosecond/1","type":"function","doc":"Returns given nanoseconds in internal Membrane time units. Inlined by the compiler."},{"ref":"Membrane.Time.html#nanoseconds/1","title":"Membrane.Time.nanoseconds/1","type":"function","doc":"The same as nanosecond/1. Inlined by the compiler."},{"ref":"Membrane.Time.html#native_unit/1","title":"Membrane.Time.native_unit/1","type":"function","doc":"Returns given native units in internal Membrane time units. Inlined by the compiler."},{"ref":"Membrane.Time.html#native_units/1","title":"Membrane.Time.native_units/1","type":"function","doc":"The same as native_unit/1. Inlined by the compiler."},{"ref":"Membrane.Time.html#os_time/0","title":"Membrane.Time.os_time/0","type":"function","doc":"Returns current POSIX time of operating system in internal Membrane time units. Inlined by the compiler."},{"ref":"Membrane.Time.html#pretty_duration/1","title":"Membrane.Time.pretty_duration/1","type":"function","doc":"Returns duration as a string with unit. Chosen unit is the biggest possible that doesn&#39;t involve precission loss. Examples iex&gt; import Membrane.Time iex&gt; 10 |&gt; milliseconds() |&gt; pretty_duration() &quot;10 ms&quot; iex&gt; 60_000_000 |&gt; microseconds() |&gt; pretty_duration() &quot;1 min&quot; iex&gt; 2 |&gt; nanoseconds() |&gt; pretty_duration() &quot;2 ns&quot;"},{"ref":"Membrane.Time.html#pretty_now/0","title":"Membrane.Time.pretty_now/0","type":"function","doc":"Returns current time in pretty format (currently iso8601), as string Uses system_time/0 under the hood."},{"ref":"Membrane.Time.html#second/1","title":"Membrane.Time.second/1","type":"function","doc":"Returns given seconds in internal Membrane time units. Inlined by the compiler."},{"ref":"Membrane.Time.html#seconds/1","title":"Membrane.Time.seconds/1","type":"function","doc":"The same as second/1. Inlined by the compiler."},{"ref":"Membrane.Time.html#system_time/0","title":"Membrane.Time.system_time/0","type":"function","doc":"Returns current time of Erlang VM based on System.system_time/0 in internal Membrane time units. Inlined by the compiler."},{"ref":"Membrane.Time.html#to_code/1","title":"Membrane.Time.to_code/1","type":"function","doc":"Returns quoted code producing given amount time. Chosen unit is the biggest possible that doesn&#39;t involve precission loss. Examples iex&gt; import Membrane.Time iex&gt; 10 |&gt; milliseconds() |&gt; to_code() |&gt; Macro.to_string() quote do 10 |&gt; Membrane.Time.milliseconds() end |&gt; Macro.to_string() iex&gt; 60_000_000 |&gt; microseconds() |&gt; to_code() |&gt; Macro.to_string() quote do 1 |&gt; Membrane.Time.minutes() end |&gt; Macro.to_string() iex&gt; 2 |&gt; nanoseconds() |&gt; to_code() |&gt; Macro.to_string() quote do 2 |&gt; Membrane.Time.nanoseconds() end |&gt; Macro.to_string()"},{"ref":"Membrane.Time.html#to_code_str/1","title":"Membrane.Time.to_code_str/1","type":"function","doc":"Returns string representation of result of to_code/1."},{"ref":"Membrane.Time.html#to_datetime/1","title":"Membrane.Time.to_datetime/1","type":"function","doc":"Returns time as a DateTime struct. TimeZone is set to UTC. Inlined by the compiler."},{"ref":"Membrane.Time.html#to_days/1","title":"Membrane.Time.to_days/1","type":"function","doc":"Returns time in days. Rounded using Kernel.round/1 Inlined by the compiler."},{"ref":"Membrane.Time.html#to_hours/1","title":"Membrane.Time.to_hours/1","type":"function","doc":"Returns time in hours. Rounded using Kernel.round/1 Inlined by the compiler."},{"ref":"Membrane.Time.html#to_iso8601/1","title":"Membrane.Time.to_iso8601/1","type":"function","doc":"Returns time as a iso8601 string. Inlined by the compiler."},{"ref":"Membrane.Time.html#to_microseconds/1","title":"Membrane.Time.to_microseconds/1","type":"function","doc":"Returns time in microseconds. Rounded using Kernel.round/1 Inlined by the compiler."},{"ref":"Membrane.Time.html#to_milliseconds/1","title":"Membrane.Time.to_milliseconds/1","type":"function","doc":"Returns time in milliseconds. Rounded using Kernel.round/1 Inlined by the compiler."},{"ref":"Membrane.Time.html#to_minutes/1","title":"Membrane.Time.to_minutes/1","type":"function","doc":"Returns time in minutes. Rounded using Kernel.round/1 Inlined by the compiler."},{"ref":"Membrane.Time.html#to_nanoseconds/1","title":"Membrane.Time.to_nanoseconds/1","type":"function","doc":"Returns time in nanoseconds. Rounded using Kernel.round/1 Inlined by the compiler."},{"ref":"Membrane.Time.html#to_native_units/1","title":"Membrane.Time.to_native_units/1","type":"function","doc":"Returns time in system native units. Rounded using Kernel.round/1 Inlined by the compiler."},{"ref":"Membrane.Time.html#to_seconds/1","title":"Membrane.Time.to_seconds/1","type":"function","doc":"Returns time in seconds. Rounded using Kernel.round/1 Inlined by the compiler."},{"ref":"Membrane.Time.html#vm_time/0","title":"Membrane.Time.vm_time/0","type":"function","doc":"Returns current Erlang VM system time based on System.system_time/0 in internal Membrane time units. It is the VM view of the os_time/0. They may not match in case of time warps. It is not monotonic. Inlined by the compiler."},{"ref":"Membrane.Time.html#t:native_t/0","title":"Membrane.Time.native_t/0","type":"type","doc":""},{"ref":"Membrane.Time.html#t:non_neg_t/0","title":"Membrane.Time.non_neg_t/0","type":"type","doc":""},{"ref":"Membrane.Time.html#t:t/0","title":"Membrane.Time.t/0","type":"type","doc":""},{"ref":"readme.html","title":"Membrane Multimedia Framework","type":"extras","doc":"Membrane Multimedia Framework Membrane Framework - if you asked me what it is, I have no idea. - Jose Valim in ElixirConf 2018 Keynote Multimedia processing framework that focuses on reliability, concurrency and scalability. An easy to use abstraction layer for assembling mostly server-side applications that have to consume, produce or process multimedia streams. It puts reliability over amount of features. It is written in Elixir + C with outstanding Erlang VM underneath that gives us a rock solid and battle tested foundation."},{"ref":"readme.html#membrane-core","title":"Membrane Multimedia Framework - Membrane Core","type":"extras","doc":"This package provides core of the Membrane Multimedia Framework."},{"ref":"readme.html#installation","title":"Membrane Multimedia Framework - Installation","type":"extras","doc":"Add the following line to your deps in mix.exs. Run mix deps.get. {:membrane_core, &quot;~&gt; 0.3.0&quot;}"},{"ref":"readme.html#getting-started","title":"Membrane Multimedia Framework - Getting started","type":"extras","doc":"To get familiar with basic concepts and build your first application using Membrane Framework, visit Membrane Guide. API documentation is available at HexDocs"},{"ref":"readme.html#support-and-questions","title":"Membrane Multimedia Framework - Support and questions","type":"extras","doc":"If you have any problems with Membrane Framework feel free to contact us on the mailing list, Discord or via e-mail."},{"ref":"readme.html#copyright-and-license","title":"Membrane Multimedia Framework - Copyright and License","type":"extras","doc":"Copyright 2018, Software Mansion Licensed under the Apache License, Version 2.0"}]